// Copyright (C) 1997- 1999 Jan Kreft <Kreft@cardiff.ac.uk>

#import <simtools.h>
#import <math.h>
#import "Base.h"
#import "ResourceQuad.h"
#import "LandRoot.h"
#import "GeckoControlSwarm.h"
#import "Critter.h"
#import "Bacillus.h"
#import "Diffuse.h"
#import "RandSource.h"

static GRate dtmin;    // smallest timestep of all dt[i] is the one to be used
static GRate dtfix;    // fixed timestep in order to make maxDelta/dt... independent of changes in dtmin
static GRate maxDelta; // for conversion criterion
static timeval_t lastDtminChange; // store time step of last change of dtmin as a response to instability
static BOOL haveEps = FALSE; // do we have eps at all?
static GRate KaNH3;  // Acidity constant for Ammonia
static GRate KbHNO2; // Basicity constant for Nitrite

@implementation DiffuseQuad

-init {
  [super init];
  type		        = diffuse;
  resource              = 1; // We don't deal with resources, but need some meaningful value for LandRoot

  haveInitMcr           = 0;
  numSubs               = 1;
  numSpecies            = 1;

  extConcn0             = 1;
  extConcn1             = 1;
  extConcn2             = 1;
  extConcn3             = 1;
  diffC0                = 10000;
  diffC1                = 10000;
  diffC2                = 10000;
  diffC3                = 10000;

  extConcn              = 0;
  diffC                 = 0;
  D                     = 0;
  Ar                    = 0;
  dt                    = 0;
  max                   = 0;
  maxlast               = 0;

  cellSize              = 1.0;
  cellsX                = 11;
  cellsY                = 11;
  boundaryLayer         = 0;
  carrierLayer          = 0;
  sliceThickness        = 1.0;
  plate                 = FALSE;
  relaxed               = TRUE;
  relaxedSub            = 0;
  stable                = TRUE;
  front                 = 0;
  countDiff             = 0;
  totDiffSteps          = 0;
  numColors             = 10;
  highColorDensity 	= 1.0;
  lowColorDensity 	= 0.00001;
  s                     = 0;
  tmp                   = 0;
  delta                 = 0;
  r                     = 0;
  x                     = 0;
  g                     = 0;
  xtot                  = 0;
  xbac                  = 0;
  xeps                  = 0;
  por                   = 0;
  d                     = 0;
  c                     = 0;
  cs                    = 0;
  colorDensity 	        = 0;
  colorIds 		= 0;
  cellVol 		= 0.0;
  bac1List 		= 0;
  bac2List 		= 0;
  epsList 		= 0;

  return self;
}

-setHaveInitMcr {haveInitMcr = 0; return self;} // Site want's to know

+create {
  id result;
  result = [[DiffuseQuad createBegin: globalZone] createEnd];
  [result init];
  return result;
}

-addSiteParams {
  [siteDef addParam: "NumSubs" Vary: 0 IValue: numSubs
	   Message: M(setNumSubs:) Owner: self];
  [siteDef addParam: "NumSpecies" Vary: 0 IValue: numSpecies
	   Message: M(setNumSpecies:) Owner: self];
  [siteDef addParam: "CellsX" Vary: 0 IValue: cellsX
	   Message: M(setCellsX:) Owner: self];
  [siteDef addParam: "CellsY" Vary: 0 IValue: cellsY
	   Message: M(setCellsY:) Owner: self];
  [siteDef addParam: "CellSize" Vary: 0 DValue: cellSize
	   Message: M(setCellSize:) Owner: self];
  [siteDef addParam: "Plate" Vary: 0 IValue: plate
	   Message: M(setPlate:) Owner: self];
  [siteDef addParam: "BoundaryLayer" Vary: 0 IValue: boundaryLayer
	   Message: M(setBoundaryLayer:) Owner: self];
  [siteDef addParam: "CarrierLayer" Vary: 0 IValue: carrierLayer
	   Message: M(setCarrierLayer:) Owner: self];
  [siteDef addParam: "SliceThickness" Vary: 0 DValue: sliceThickness
	   Message: M(setSliceThickness:) Owner: self];
  [siteDef addParam: "DiffC0" Vary: 0 DValue: diffC0
	   Message: M(setDiffC0:) Owner: self];
  [siteDef addParam: "DiffC1" Vary: 0 DValue: diffC1
	   Message: M(setDiffC1:) Owner: self];
  [siteDef addParam: "DiffC2" Vary: 0 DValue: diffC2
	   Message: M(setDiffC2:) Owner: self];
  [siteDef addParam: "DiffC3" Vary: 0 DValue: diffC3
	   Message: M(setDiffC3:) Owner: self];
  [siteDef addParam: "ExtConcn0" Vary: 0 DValue: extConcn0
	   Message: M(setExtConcn0:) Owner: self];
  [siteDef addParam: "ExtConcn1" Vary: 0 DValue: extConcn1
	   Message: M(setExtConcn1:) Owner: self];
  [siteDef addParam: "ExtConcn2" Vary: 0 DValue: extConcn2
	   Message: M(setExtConcn2:) Owner: self];
  [siteDef addParam: "ExtConcn3" Vary: 0 DValue: extConcn3
	   Message: M(setExtConcn3:) Owner: self];
  [siteDef addParam: "NumColors" Vary: 0 IValue: numColors
	   Message: M(setNumColors:) Owner: self];
  [siteDef addParam: "HighColorDensity" Vary: 0 DValue: highColorDensity
	   Message: M(setHighColorDensity:) Owner: self];
  [siteDef addParam: "LowColorDensity" Vary: 0 DValue: lowColorDensity
	   Message: M(setLowColorDensity:) Owner: self];
  
  return self;
}

-landGrandma: (Site*) site {
  // one default parameterized DiffuseQuad--relies on defaults in init.
  // Grandma has no lattice--work areas created by copymom.
  
  siteDef = site;
  
  [self addSiteParams];
  
  [self setMcrLX: 0 LY: 0
	UX: (cellsX * cellSize) 
	UY: (cellsY * cellSize)];
  return self;
}

-(GRate**) allocFloatLattice {
  // lattice[xi][yi]
  GRate** l;
  int i, j;
  
  l = (GRate**) [globalZone alloc: (cellsX) * sizeof(GRate*)];
  for (i=0; i < cellsX; i++)
    l[i] = (GRate*) [globalZone alloc: (cellsY) * sizeof(GRate)];

  return l;
}

-(char**) allocCharLattice {
  // lattice[xi][yi]
  char** l;
  int i, j;
  
  l = (char**) [globalZone alloc: (cellsX) * sizeof(char*)];
  for (i=0; i < cellsX; i++)
    l[i] = (char*) [globalZone alloc: (cellsY) * sizeof(char)];

  return l;
}

-fillLattice: (GRate**) which WithFloats: (GRate) fc {
  // which is the lattice to fill?
  // fc = fill concn
  int i, j;
  
  for (i=0; i < cellsX; i++)
    for (j=0; j < cellsY; j++)
      which[i][j] = fc;
  
  return self;
}

-fillLattice: (char**) which WithChars: (char) fv {
  // fv = fill value
  int i, j;
  
  for (i=0; i < cellsX; i++)
    for (j=0; j < cellsY; j++)
      which[i][j] = fv;
  
  return self;
}

-printFloatLattice: (GRate**) which {
  // which is the lattice to print?
  int i, j;
  
  for (j=0; j < cellsY; j++) {
    for (i=0; i < cellsX; i++) {
      printLogD("%3.1E ", which[i][j]);
    }
    printLog("\n");
  }
  return self;
}

-printCharLattice: (char**) which {
  // which is the lattice to print?
  int i, j;
  
  for (j=0; j < cellsY; j++) {
    for (i=0; i < cellsX; i++) {
      printLog("%d ", which[i][j]);
    }
    printLog("\n");
  }
  return self;
}

-initColors
    
{
    int i, colorstep;
    GRate colDelta;
    GRate divisor;

    if (numColors > NumDiffuseColors)
        numColors = NumDiffuseColors; 
    
    // allocate the space
    colorDensity = [[self getZone] alloc: numColors * sizeof(GRate)];
    colorIds 	 = [[self getZone] alloc: numColors * sizeof(int)];
    
    // calculate the thresholds

    // Display directives - min #colors = 2, max determined by color manager,
    // which doesn't exist yet (970424).  Densities between high and low are
    // divvied up in a sorta-orders-of-magnitude scale, where 
    // subrange top/bottom = pow(highCD/lowCD, 1.0/(numColors)).
    // Ex. high 1.0, low .00001, 5 orders of magnitude:
    // numColors 5: 1.0, 0.1, 0.01, 0.001, 0.0001, 0.00001
    // numColors 10: 1.0, 0.317, 0.1, 0.0317, 0.01, 0.00317, ....
    // densities < low are black, > high are high.  Black isn't one of numColors.

    // This is the above described logarithmic scheme
    //colDelta = highColorDensity / lowColorDensity;
    //divisor = pow(colDelta, (1.0 / numColors));
    //colorDensity[0] = highColorDensity / divisor;
    //for (i = 1; i < numColors - 1; i++)
    //    colorDensity[i] = colorDensity[i-1] / divisor;
    //colorDensity[numColors - 1] = 0.0;

    // Here is a linear scheme for our normalized substrate field, from 1.4 to 0 since products can be > 1
    colDelta = 1.4 / numColors;
    colorDensity[0] = 1.4 - colDelta;
    for (i = 1; i < numColors - 1; i++)
        colorDensity[i] = colorDensity[i-1] - colDelta;
    colorDensity[numColors - 1] = 0.0;
    
    // dole out available colors equably
    colorstep = 1;
    if (numColors < NumDiffuseColors) { 
        colorstep = (int) (NumDiffuseColors / (0.5 + numColors));
        if ((colorstep * numColors) > NumDiffuseColors)
            colorstep--;
    }
    colorIds[0] = HighDiffuseColor;
    for (i = 1; i < numColors - 1; i++)
        colorIds[i] = colorIds[i-1] + colorstep;
    colorIds[numColors - 1] = LowDiffuseColor;
    
    //[self printColorThresholds];
    return self;
}

-(int) getColorCellX: (int) xi Y: (int) yi {
  if (c[xi][yi] == 2) return [self getColorDensity: -1.0 ]; // make the carrier a color out of diffuse color range
  return [self getColorDensity: s[0][xi][yi] ]; // display oxygen for now
}

-(int) getColorDensity: (GRate) den {

  int i;
  
  if (den < 0) return GroundRichColor; // some kind of brown

  for (i=0; i < numColors; i++) {
    if (den >= colorDensity[i]) {
      return colorIds[i];
    }
  }
  return 0;    
}

-copymom
    // self=mom, returns copy
    // this actually allocates all the workspace--grandmas nil
{
    DiffuseQuad* newbie;

    // if exists, return existing copy

    newbie = [DiffuseQuad create]; // create calls init
    [self copyTo: newbie];
    return newbie;
}

-copyTo: (DiffuseQuad*) newbie {
  int i, j;

  newbie->siteDef  		= siteDef;
  newbie->resource              = resource;

  newbie->numSubs               = numSubs;
  newbie->numSpecies            = numSpecies;
  newbie->extConcn0             = extConcn0;
  newbie->extConcn1             = extConcn1;
  newbie->extConcn2             = extConcn2;
  newbie->extConcn3             = extConcn3;
  newbie->diffC0                = diffC0;
  newbie->diffC1                = diffC1;
  newbie->diffC2                = diffC2;
  newbie->diffC3                = diffC3;
  newbie->cellSize 		= cellSize;
  newbie->cellsX  		= cellsX;
  newbie->cellsY 		= cellsY;
  newbie->boundaryLayer         = boundaryLayer;
  newbie->carrierLayer          = carrierLayer;
  newbie->sliceThickness 	= sliceThickness;
  newbie->plate                 = plate;
  newbie->relaxed               = relaxed;
  newbie->stable                = stable;
  newbie->haveInitMcr           = haveInitMcr;
  newbie->front                 = front;
  newbie->countDiff       	= countDiff;
  newbie->totDiffSteps          = totDiffSteps;
  newbie->numColors 		= numColors;
  newbie->highColorDensity 	= highColorDensity;
  newbie->lowColorDensity 	= lowColorDensity;
  [newbie initColors];
  
  newbie->cellVol 		= cellSize * cellSize * sliceThickness;

  [newbie setMcrLX: mcr->lx LY: mcr->ly 
	  UX: mcr->lx + (cellsX * cellSize) 
	  UY: mcr->ly + (cellsY * cellSize)];

  newbie->relaxedSub	= (BOOL*)  [globalZone alloc: numSubs * sizeof(BOOL)];
  newbie->extConcn    	= (GRate*) [globalZone alloc: numSubs * sizeof(GRate)];
  newbie->diffC       	= (GRate*) [globalZone alloc: numSubs * sizeof(GRate)];
  newbie->D             = (GRate*) [globalZone alloc: numSubs * sizeof(GRate)];
  newbie->dt            = (GRate*) [globalZone alloc: numSubs * sizeof(GRate)];
  newbie->Ar            = (GRate*) [globalZone alloc: numSubs * sizeof(GRate)];

  for (i=0; i < numSubs; i++)
    newbie->relaxedSub[i] = TRUE;

  newbie->extConcn[0] = extConcn0;
  newbie->extConcn[1] = extConcn1;
  newbie->extConcn[2] = extConcn2;
  newbie->extConcn[3] = extConcn3;
  newbie->diffC[0] = diffC0;
  newbie->diffC[1] = diffC1;
  newbie->diffC[2] = diffC2;
  newbie->diffC[3] = diffC3;

  // Scale our diffusion coefficients diffC[]
  for (i=0; i < numSubs; i++)
    newbie->D[i] = newbie->diffC[i] / (cellSize * cellSize * 2.0);

  // Courant condition for stability
  // Note this is our choice for dt, we could use other values,
  // with this dt it happens that diffC cancels out in D*dt
  // However, this affects only the speed of relaxation per step, not the steady state obtained,
  // because diffC is implicit in dt, so with a different diffC, we alter the number of diffSteps

  for (i=0; i < numSubs; i++)
    newbie->dt[i] = cellSize * cellSize / (4.0 * newbie->diffC[i]);
  dtmin = newbie->dt[0];
  for (i=0; i < numSubs; i++)
    if (newbie->dt[i] < dtmin) dtmin = newbie->dt[i];

  dtfix = dtmin;

  printLogD("Diffusion time step not-increased to %g ", dtmin); printLog("at %d\n", [geckoControl getCurrentTime]);
  
  [newbie recalcParams]; // recalc diffusion time step (dt) dependent coefficients      

  newbie->max           = (GRate*) [globalZone alloc: numSubs * sizeof(GRate)];
  newbie->maxlast       = (GRate*) [globalZone alloc: numSubs * sizeof(GRate)];
  for (i=0; i < numSubs; i++) {
    newbie->max[i] = 0.0;
    newbie->maxlast[i] = 0.0;
  }

  newbie->s = (GRate***) [globalZone alloc: numSubs * sizeof(GRate**)];
  for (i=0; i < numSubs; i++) newbie->s[i]  = (GRate**) [newbie allocFloatLattice];

  newbie->d = (GRate***) [globalZone alloc: numSubs * sizeof(GRate**)];
  for (i=0; i < numSubs; i++) newbie->d[i]  = (GRate**) [newbie allocFloatLattice];

  newbie->r = (GRate***) [globalZone alloc: numSubs * sizeof(GRate**)];
  for (i=0; i < numSubs; i++) newbie->r[i]  = (GRate**) [newbie allocFloatLattice];

  newbie->x = (GRate***) [globalZone alloc: numSpecies * sizeof(GRate**)];
  for (i=0; i < numSpecies; i++) newbie->x[i]  = (GRate**) [newbie allocFloatLattice];

  newbie->g = (GRate***) [globalZone alloc: numSpecies * sizeof(GRate**)];
  for (i=0; i < numSpecies; i++) newbie->g[i]  = (GRate**) [newbie allocFloatLattice];

  newbie->xtot                = [newbie allocFloatLattice];
  newbie->xbac                = [newbie allocFloatLattice];
  newbie->xeps                = [newbie allocFloatLattice];
  newbie->por                 = [newbie allocFloatLattice];
  newbie->tmp                 = [newbie allocFloatLattice];
  newbie->delta               = [newbie allocFloatLattice];
  newbie->c                   = [newbie allocCharLattice];
  newbie->cs                  = [newbie allocCharLattice];


  for (i=0; i < numSubs; i++) {
    [newbie fillLattice: (GRate**) newbie->s[i]  WithFloats: 1.0];
    [newbie fillLattice: (GRate**) newbie->d[i]  WithFloats: newbie->D[i]];
    [newbie fillLattice: (GRate**) newbie->r[i]  WithFloats: 0.0];
  }

  for (i=0; i < numSpecies; i++) {
    [newbie fillLattice: (GRate**) newbie->x[i]  WithFloats: 0.0];
    [newbie fillLattice: (GRate**) newbie->g[i]  WithFloats: 0.0];
  }

  [newbie fillLattice: (GRate**) newbie->xtot  WithFloats: 0.0];
  [newbie fillLattice: (GRate**) newbie->xbac  WithFloats: 0.0];
  [newbie fillLattice: (GRate**) newbie->xeps  WithFloats: 0.0];
  [newbie fillLattice: (GRate**) newbie->por   WithFloats: 1.0];
  [newbie fillLattice: (GRate**) newbie->tmp   WithFloats: 1.0];
  [newbie fillLattice: (GRate**) newbie->delta WithFloats: 1.0];
  [newbie fillLattice: (char**)  newbie->c     WithChars:  0];
  [newbie fillLattice: (char**)  newbie->cs    WithChars:  0];

  if (newbie->plate) { // make carrier in c and cs matrix
    for (i=0; i < rint(carrierLayer/cellSize); i++) 
      for (j=0; j < cellsY; j++) { newbie->c[i][j] = 2; newbie->cs[i][j] = 2; }
  }

  // Allocate the vectors for tridag
  newbie->aa  = (GRate*) [globalZone alloc: (cellsX > cellsY ? cellsX+1 : cellsY+1) * sizeof(GRate)];
  newbie->bb  = (GRate*) [globalZone alloc: (cellsX > cellsY ? cellsX+1 : cellsY+1) * sizeof(GRate)];
  newbie->cc  = (GRate*) [globalZone alloc: (cellsX > cellsY ? cellsX+1 : cellsY+1) * sizeof(GRate)];
  newbie->rr  = (GRate*) [globalZone alloc: (cellsX > cellsY ? cellsX+1 : cellsY+1) * sizeof(GRate)];
  newbie->uu  = (GRate*) [globalZone alloc: (cellsX > cellsY ? cellsX+1 : cellsY+1) * sizeof(GRate)];
  newbie->gam = (GRate*) [globalZone alloc: (cellsX > cellsY ? cellsX+1 : cellsY+1) * sizeof(GRate)];
  newbie->hh  = (GRate*) [globalZone alloc: (cellsX > cellsY ? cellsX+1 : cellsY+1) * sizeof(GRate)];
  newbie->vv  = (GRate*) [globalZone alloc: (cellsX > cellsY ? cellsX+1 : cellsY+1) * sizeof(GRate)];
  
  newbie->bac1List = [List create: [self getZone]];
  newbie->bac2List = [List create: [self getZone]];
  newbie->epsList  = [List create: [self getZone]];

  return self;
}

// Bacillus needs to know the size of the carrier
-(GCoord) getCarrierLayer {
  if (plate) return carrierLayer;
  return 0.0;
}

// Bacillus needs to know whether we have periodic y boundaries for cell movement
-(BOOL) getPeriodicY {
  if (plate) return TRUE;
  return FALSE;
}

// Bacillus needs to know the length of the x coordinate
-(GCoord) getLengthX {
  return (cellSize * cellsX);
}

// Bacillus needs to know the length of the y coordinate
-(GCoord) getLengthY {
  return (cellSize * cellsY);
}

// Bacillus needs to know the length of the z coordinate (sliceThickness)
-(GCoord) getLengthZ {
  return sliceThickness;
}

// Bacillus needs to know the cellSize
-(GCoord) getCellSize {
  return cellSize;
}

-(int) getNumSubs {
  return numSubs;
}

-(int) getNumSpecies {
  return numSpecies;
}

// Bacillus needs to know how much it overlaps with carrier
-(GCoord) getOverlapWithCarrierAtX: (GCoord) xi Y: (GCoord) yi {
  // remember agent coords are floats, xi is "left edge" of bacterial cell (x - distRadius), yi is center

  if (plate) { // got carrier at all?
    if (xi - carrierLayer >= 0) { // no overlap
      return 0.0;
    }
    return (-xi + carrierLayer); // return overlap as positive value
  }
  return 0.0;
}

// Bacillus needs to find out for normalising kms and kis
-(GRate) getExtConcn: (BacillusSub) which {
  return extConcn[which];
}

-(GRate) getConcnSub: (BacillusSub) this X: (GCoord) xi Y: (GCoord) yi {
// We need a getConcn message for access from the outside (Bacilli)
// Internally, we don't use messages for access, we use s[sub][xi][yi]

  // remember agent coords are floats
  return s[this][(int) (xi / cellSize)][(int) (yi / cellSize)];
}

-doleOut: (GRate) what To: (GRate**) latt X: (GCoord) xpos Y: (GCoord) ypos Radius: (GCoord) radius {

  int xi, yi;
  int xm, xp, ym, yp;
  GArea bacArea;
  GCoord rad, blx, bly, bux, buy, llx, lly, lux, luy;
  GRate quant;

  xi  = (int) (xpos / cellSize); // agent coord converted to integer lattice position, that's why i
  yi  = (int) (ypos / cellSize);

  // calc lattice neighbours
  if (plate) {
    if (yi==0) ym=cellsY-1; else ym=yi-1;
    if (yi==cellsY-1) yp=0; else yp=yi+1;
  } else {
    ym=yi-1;
    yp=yi+1;
  }
  xm=xi-1;
  xp=xi+1;
  
  // area to be shared
  rad = CubertPiOver6 * radius; // radius of cube with equivalent volume
  if ( (2*rad) > cellSize) printLogD("Warning, cube > cellSize, with %g\n", (2*rad) );
  bacArea = 4.0 * rad * rad;
  // amount to be shared
  quant = what / bacArea;

  // calc windows
  blx = xpos - rad; // lower x of bacillus equiv-volume cube
  bly = ypos - rad; // lower y
  bux = xpos + rad; // upper x
  buy = ypos + rad; // upper y
  llx = xi * cellSize;  // lower x of lattice element = grid node
  lly = yi * cellSize;  // lower y
  lux = llx + cellSize; // upper x
  luy = lly + cellSize; // upper y
  
  // calc the compass needle case
  if (blx < llx) { // to the left
    if (bly < lly) { // to the bottom
      // SW case
      latt[xi][yi] += quant * (bux - llx) * (buy - lly); // C
      latt[xm][yi] += quant * (llx - blx) * (buy - lly); // W
      latt[xm][ym] += quant * (llx - blx) * (lly - bly); // SW
      latt[xi][ym] += quant * (bux - llx) * (lly - bly); // S
    }
    else if (buy > luy) { // or to the top, relies on cellSize > cube
      // NW case
      latt[xi][yi] += quant * (bux - llx) * (luy - bly); // C
      latt[xi][yp] += quant * (bux - llx) * (buy - luy); // N
      latt[xm][yp] += quant * (llx - blx) * (buy - luy); // NW
      latt[xm][yi] += quant * (llx - blx) * (luy - bly); // W
    }
    else { // neither => middle
      // W case
      latt[xi][yi] += quant * (bux - llx) * (buy - bly); // C
      latt[xm][yi] += quant * (llx - blx) * (buy - bly); // W
    }
  }
  else if (bux > lux) { // to the right
    if (bly < lly) { // to the bottom
      // SE case
      latt[xi][yi] += quant * (lux - blx) * (buy - lly); // C
      latt[xi][ym] += quant * (lux - blx) * (lly - bly); // S
      latt[xp][ym] += quant * (bux - lux) * (lly - bly); // SE
      latt[xp][yi] += quant * (bux - lux) * (buy - lly); // E
    }
    else if (buy > luy) { // or to the top
      // NE case
      latt[xi][yi] += quant * (lux - blx) * (luy - bly); // C
      latt[xp][yi] += quant * (bux - lux) * (luy - bly); // E
      latt[xp][yp] += quant * (bux - lux) * (buy - luy); // NE
      latt[xi][yp] += quant * (lux - blx) * (buy - luy); // N
    }
    else { // neither => middle
      // E case
      latt[xi][yi] += quant * (lux - blx) * (buy - bly); // C
      latt[xp][yi] += quant * (bux - lux) * (buy - bly); // E
    }
  }
  else { // neither => middle
    if (bly < lly) { // to the bottom
      // S case
      latt[xi][yi] += quant * (bux - blx) * (buy - lly); // C
      latt[xi][ym] += quant * (bux - blx) * (lly - bly); // S
    }
    else if (buy > luy) { // or to the top
      // N case
      latt[xi][yi] += quant * (bux - blx) * (luy - bly); // C
      latt[xi][yp] += quant * (bux - blx) * (buy - luy); // N
    }
    else { // neither => middle, middle
      // C case
      latt[xi][yi] += quant * bacArea; // C
    }
  }
  
  return self;
}

-(GRate) avgSub: (BacillusSub) this X: (GCoord) xpos Y: (GCoord) ypos Radius: (GCoord) radius {
  // Average the subConcn in lattice cells we sit on, according to area percentage
  int xi, yi;
  int xm, xp, ym, yp;
  GArea bacArea;
  GCoord rad, blx, bly, bux, buy, llx, lly, lux, luy;
  GRate quant = 0.0; // averaged subConcn

  xi  = (int) (xpos / cellSize); // agent coord converted to integer lattice position, that's why i
  yi  = (int) (ypos / cellSize);

  // calc lattice neighbours
  if (plate) {
    if (yi==0) ym=cellsY-1; else ym=yi-1;
    if (yi==cellsY-1) yp=0; else yp=yi+1;
  } else {
    ym=yi-1;
    yp=yi+1;
  }
  xm=xi-1;
  xp=xi+1;
  
  // area to be shared
  rad = CubertPiOver6 * radius; // radius of cube with equivalent volume
  //if ( (2*rad) > cellSize) printLogD("Warning, cube > cellSize, with %g\n", (2*rad) );
  bacArea = 4.0 * rad * rad;

  // calc windows
  blx = xpos - rad; // lower x of bacillus equiv-volume cube
  bly = ypos - rad; // lower y
  bux = xpos + rad; // upper x
  buy = ypos + rad; // upper y
  llx = xi * cellSize;  // lower x of lattice element = grid node
  lly = yi * cellSize;  // lower y
  lux = llx + cellSize; // upper x
  luy = lly + cellSize; // upper y
  
  // calc the compass needle case
  if (blx < llx) { // to the left
    if (bly < lly) { // to the bottom
      // SW case
      quant  = s[this][xi][yi] * (bux - llx) * (buy - lly) / bacArea; // C
      quant += s[this][xm][yi] * (llx - blx) * (buy - lly) / bacArea; // W
      quant += s[this][xm][ym] * (llx - blx) * (lly - bly) / bacArea; // SW
      quant += s[this][xi][ym] * (bux - llx) * (lly - bly) / bacArea; // S
    }
    else if (buy > luy) { // or to the top, relies on cellSize > cube
      // NW case
      quant  = s[this][xi][yi] * (bux - llx) * (luy - bly) / bacArea; // C
      quant += s[this][xi][yp] * (bux - llx) * (buy - luy) / bacArea; // N
      quant += s[this][xm][yp] * (llx - blx) * (buy - luy) / bacArea; // NW
      quant += s[this][xm][yi] * (llx - blx) * (luy - bly) / bacArea; // W
    }
    else { // neither => middle
      // W case
      quant  = s[this][xi][yi] * (bux - llx) * (buy - bly) / bacArea; // C
      quant += s[this][xm][yi] * (llx - blx) * (buy - bly) / bacArea; // W
    }
  }
  else if (bux > lux) { // to the right
    if (bly < lly) { // to the bottom
      // SE case
      quant  = s[this][xi][yi] * (lux - blx) * (buy - lly) / bacArea; // C
      quant += s[this][xi][ym] * (lux - blx) * (lly - bly) / bacArea; // S
      quant += s[this][xp][ym] * (bux - lux) * (lly - bly) / bacArea; // SE
      quant += s[this][xp][yi] * (bux - lux) * (buy - lly) / bacArea; // E
    }
    else if (buy > luy) { // or to the top
      // NE case
      quant  = s[this][xi][yi] * (lux - blx) * (luy - bly) / bacArea; // C
      quant += s[this][xp][yi] * (bux - lux) * (luy - bly) / bacArea; // E
      quant += s[this][xp][yp] * (bux - lux) * (buy - luy) / bacArea; // NE
      quant += s[this][xi][yp] * (lux - blx) * (buy - luy) / bacArea; // N
    }
    else { // neither => middle
      // E case
      quant  = s[this][xi][yi] * (lux - blx) * (buy - bly) / bacArea; // C
      quant += s[this][xp][yi] * (bux - lux) * (buy - bly) / bacArea; // E
    }
  }
  else { // neither => middle
    if (bly < lly) { // to the bottom
      // S case
      quant  = s[this][xi][yi] * (bux - blx) * (buy - lly) / bacArea; // C
      quant += s[this][xi][ym] * (bux - blx) * (lly - bly) / bacArea; // S
    }
    else if (buy > luy) { // or to the top
      // N case
      quant  = s[this][xi][yi] * (bux - blx) * (luy - bly) / bacArea; // C
      quant += s[this][xi][yp] * (bux - blx) * (buy - luy) / bacArea; // N
    }
    else { // neither => middle, middle
      // C case
      quant = s[this][xi][yi]; // C
    }
  }
  
  return quant;
}

-(GRate) calcPorosity {
  // upon request by Bacillus, calc Porosity stats, convenience method

  // To do: minPoros depends heavily on resolution of the porosity lattice, below 4 microns, the trace is really
  // to noisy, above it, we average too much and would fail to detect local overcrowding.
  // Optimal solution would be a porosity lattice with cellSize independent of the substrate etc. lattice, with,
  // say, 4 micron fixed resolution (overhanging edges maybe if total system size not divisible by 4).

  int i, j;
  int count = 0;
  GRate sum = 0.0;
  GRate sumsqr = 0.0;
  GRate poros;
  GRate mass;
  GRate porosMin = 1.0;
  GCoord xf, yf, radius;
  Quad* q;
  Quad* mobileRoot = [[geckoControl getModelSwarm] getMobileRoot];

  // calc xbac matrix

  // zero the matrix first
  [self fillLattice: (GRate**) xbac WithFloats: 0.0];

  //traverse the agent quadding tree and read info from each agent
  q = [mobileRoot pickWin: mobileRoot->mcr Root: mobileRoot Invisibles: 0];
  
  while (q) {
    
    if ([q->data getProcess] > 0) { // if not Eps
      
      // get the agents position, note that agents x, y coordinates are GCoord, i.e. floats
      xf  = [q->data getGX]; // agent coords, f for float
      yf  = [q->data getGY];
      // get the agents radius
      radius = [q->data getRadius];
      // get the agents mass and dole it out over the lattice grids covered, for each lattice separately
      mass = [q->data getDryMass];

      [self doleOut: (GRate) mass To: (GRate**) xbac X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
      //xbac[(int)(xf / cellSize)][(int)(yf / cellSize)] += mass;
      
    }
    q = [q pickWin: mobileRoot->mcr Root: mobileRoot Invisibles: 0];
  }
  
  // calc porosity
  
  for (i=0; i < cellsX; i++) {
    for (j=0; j < cellsY; j++) {
      // Porosity is 1 - (biovolume / total volume)
      if (xbac[i][j] > EPSILON) { // We want to focus on the inside of the colony/biofilm
	poros = 1.0 - (xbac[i][j]/290.0) / cellVol;
	sum    += poros;
	sumsqr += poros * poros;
	count++; 
      } else {
	poros = 1.0;
      }
      if (poros < porosMin) porosMin = poros;
    }
  }
  
  printPor("%d\t", [geckoControl getCurrentTime]);
  printPorD("%g\t", sum/count); // mean
  printPorD("%g\t", sqrt( (sumsqr/count) - (sum/count) * (sum/count) ) ); // SD
  printPorD("%g\n", porosMin);        // minimum
  return porosMin;
}

-update_x_Matrices {
  // We want to sum the biomass existing in every lattice cell
  // xtot is total biomass of all cells and eps, that is, cells and their products
  // xbac (x total) is any cellular biomass, not species-specific, useful for porosity
  // xeps is total mass of eps
  // x[i] is species and therefore process specific
  
  BOOL overBoard = FALSE;
  int p, i;
  GCoord xf, yf, radius;
  GRate mass, grate;
  Quad* q;
  Quad* mobileRoot = [[geckoControl getModelSwarm] getMobileRoot];

  // zero the matrices
  [self fillLattice: (GRate**) xtot WithFloats: 0.0];
  [self fillLattice: (GRate**) xbac WithFloats: 0.0];
  [self fillLattice: (GRate**) xeps WithFloats: 0.0];
  for (i=0; i < numSpecies; i++)
    [self fillLattice: (GRate**) x[i] WithFloats: 0.0];

  //traverse the agent quadding tree and read info from each agent
  q = [mobileRoot pickWin: mobileRoot->mcr Root: mobileRoot Invisibles: 0];

  while (q) {
    
    // get the agents position, note that agents x, y coordinates are GCoord, i.e. floats
    xf  = [q->data getGX]; // agent coords, f for float
    yf  = [q->data getGY];
    // get the agents radius
    radius = [q->data getRadius];
    
    // check whether our agent went over board, if so, stop the ship
    if (plate) {
      if (((xf-radius)<0) || ((xf+radius)>(cellsX*cellSize)) || (yf<0) || (yf>(cellsY*cellSize)))
	overBoard = TRUE;
    }
    else {
      if (((xf-radius)<0) || ((xf+radius)>(cellsX*cellSize)) || ((yf-radius)<0) || ((yf+radius)>(cellsY*cellSize)))
	overBoard = TRUE;
    }
    if (overBoard) {
      printLogD("Warning, agent over board at %g ", xf);
      printLogD("%g => halt the ship!\n", yf);
      [geckoControl setWantToStop: TRUE];
      overBoard = FALSE;
    } else {

      // get the agents mass and dole it out over the lattice grids covered, for each lattice separately
      mass = [q->data getDryMass];
      
      // xtot matrix
      [self doleOut: (GRate) mass To: (GRate**) xtot X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
      //xtot[(int)(xf / cellSize)][(int)(yf / cellSize)] += mass;
      
      // ask for the process carried out by this particular species
      p = (int) [q->data getProcess]; // one species does one process, species must know what they do, Eps returns -1
      
      if (p > 0) { // if bacterium
      	[self doleOut: (GRate) mass To: (GRate**) xbac X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
	//xbac[(int)(xf / cellSize)][(int)(yf / cellSize)] += mass;
	// different indexing, process is [1, 2, ...] but x is [0, 1, ...]
	[self doleOut: (GRate) mass To: (GRate**) x[p-1] X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
	//x[p-1][(int)(xf / cellSize)][(int)(yf / cellSize)] += mass;

	// get the agents growth rate and dole it out over the lattice grids covered
	grate = [q->data getGrowthRate];
	[self doleOut: (GRate) grate To: (GRate**) g[p-1] X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
	//g[p-1][(int)(xf / cellSize)][(int)(yf / cellSize)] += grate;
      }
      else if (p == -1) // if eps
	[self doleOut: (GRate) mass To: (GRate**) xeps X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
        //xeps[(int)(xf / cellSize)][(int)(yf / cellSize)] += mass;
    }
    q = [q pickWin: mobileRoot->mcr Root: mobileRoot Invisibles: 0];
  }
  return self;
}

-update_c_Matrix { // carrier = 2, biofilm = 1, all liquid = 0
  // loop through the biomass matrix xtot to see where we have biomass, rest is liquid unless carrier (fixed from start)

  int i, j;

  for (i=0; i < cellsX; i++) {
    for (j=0; j < cellsY; j++) {
      if (c[i][j] != 2) // re-calc if not carrier
	c[i][j] = (xtot[i][j] > 0.0) ? 1 : 0; // set to 1 where biomass, else zero
    }
  }
  return self;
}

-(int) findFront { // find the highest peak of the biomass, called front

  int i, j;

  for (i=cellsX-1; i >= 0; i--) { // start from top and go down
    for (j=0; j < cellsY; j++) {
      if (c[i][j] == 1) return i;
    }
  }
  return 0;
}

-update_cs_Matrix { // carrier = 2, (biofilm + boundary layer) = 1, bulk liquid = 0
  // recalc the matrix where and how to calc diffusion
  // has to be redone after each growth step, i.e. when the biomass changes

  int i, j;
  int top;

  if (plate) {
    front = [self findFront]; // requires updated c matrix
    top = front + boundaryLayer + 1;
    if (top > cellsX-1) {
      top = cellsX-1;
      printLog("Warning, front over the top: %d will stop sim!\n", top);
      [geckoControl setWantToStop: TRUE];
    }
  }

  // if (submerged) cs[i][j] = 0 outside colonyRadius + colonyBoundaryLayer

  for (i=0; i < cellsX; i++) {
    for (j=0; j < cellsY; j++) {
      if (c[i][j] == 2) cs[i][j] = 2; // if carrier, set to 2 in any case
      else {
	if (plate) {
	  if (i >= top) cs[i][j] = 0; // bulk liquid on top of the biofilm + boundaryLayer
	  else cs[i][j] = 1;
	} else {
	  cs[i][j] = 1;
	  if ((i == 0) || (j == 0) || (i == cellsX-1) || (j == cellsY-1)) cs[i][j] = 0; // if border
	}
      }
    }
  }
  return self;
}

-update_d_Matrices { 
  // loop through the biomass matrix xbac to calculate the fraction of biovolume and effective diffusion coefficient
  // 290 is the dryMass density of the cells, should use a param here
  // If density of agents differs, biomass and biovolume are not interchangeable anymore
  // Note that we don't smooth biomass density, we count a cell's biomass where it's center is

  // Calculation of the effective diffusion coefficient in biofilms is discussed in the Thesis of Evelien Beuling,
  // Amsterdam. Here we use Fricke's formula with a shape factor of 2.

  // 1.0 / (D*dt) is a convenient substitute for D, since D always used in this combination in adi

  int i, j, k;
  GRate bioVolFrac;

  for (i=0; i < cellsX; i++) {
    for (j=0; j < cellsY; j++) {
      // volume fraction is biovolume / total volume
      bioVolFrac = (xbac[i][j]/290.0) / cellVol;
      por[i][j] = 1.0 - bioVolFrac;
      // The closest attainable packing of spheres gives a volume fraction of 0.74
      if (bioVolFrac > 0.74) {
	printLogD("Warning, volume fraction is %4.2f ", bioVolFrac);
	printLog("in %d %d at %d\n", i, j, [geckoControl getCurrentTime]);
	bioVolFrac = 0.74;
      }
      bioVolFrac = 0.0; // for evaluation of the effect of porosity
      for (k=0; k < numSubs; k++)
	d[k][i][j] = 1.0 / (((1.0-bioVolFrac)/(1.0+bioVolFrac/2.0))*D[k]*dtmin);
    }
  }
  return self;
}

-update_r_Matrices {
  // We want to know x & y position and params for our substrate foo at the given concn (we know the concn)
  // Let agents calculate the reaction rate from this and store their answer in the reaction rate matrix r
  
  int k, i, j, p;
  int xi, yi;
  GCoord xf, yf, radius;
  GRate rate;
  Quad* q;
  Quad* mobileRoot = [[geckoControl getModelSwarm] getMobileRoot];

  // zero the matrices
  for (k=0; k < numSubs; k++)
    [self fillLattice: (GRate**) r[k] WithFloats: 0.0];
  
  //traverse the agent quadding tree and read info from each agent, calculate reaction rate and store in r
  q = [mobileRoot pickWin: mobileRoot->mcr Root: mobileRoot Invisibles: 0];
  
  while (q) {
    // ask for the process carried out by this particular species
    p = (int) [q->data getProcess]; // one species does one process, species must know what they do, Eps returns -1
    
    if (p > 0) {
      
      // get the agents position, note that agents x, y coordinates are GCoord, i.e. floats
      xf  = [q->data getGX]; // agent coords, f for float
      yf  = [q->data getGY];
      xi  = (int) (xf / cellSize); // agent coord converted to integer lattice position, that's why i
      yi  = (int) (yf / cellSize);
      // get the agents radius
      radius = [q->data getRadius];
      
      switch (p) {
	
	// depending on the process, we ask for all relevant reaction rates and provide the relevant subConcns
      case 1: // Nitroso
	// Ammonia (1) consumption (rate will be negative), supply ammonia (1) and oxygen (0) concn
	rate = [q->data getRateForReaction: 1 Sub1: s[1][xi][yi] Sub2: s[0][xi][yi] ];
	[self doleOut: (GRate) rate To: (GRate**) r[1] X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
	//r[1][xi][yi] += rate;
	// Nitrite (2) production (rate will be positive), follows from stoichiometry
	[self doleOut: (GRate) -rate To: (GRate**) r[2] X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
	//r[2][xi][yi] += - rate;
	// Oxygen (0) consumption (rate will be negative), supply ammonia (1) and oxygen (0) concn
	rate = [q->data getRateForReaction: 2 Sub1: s[1][xi][yi] Sub2: s[0][xi][yi] ];
	[self doleOut: (GRate) rate To: (GRate**) r[0] X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
	//r[0][xi][yi] += rate;
	break; 
      case 2: // Nitro
	// Nitrite (2) consumption (rate will be negative), supply nitrite (2) and oxygen (0) concn
	rate = [q->data getRateForReaction: 11 Sub1: s[2][xi][yi] Sub2: s[0][xi][yi] ];
	[self doleOut: (GRate) rate To: (GRate**) r[2] X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
	//r[2][xi][yi] += rate;
	// Nitrate (3) production (rate will be positive), follows from stoichiometry
	[self doleOut: (GRate) -rate To: (GRate**) r[3] X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
	//r[3][xi][yi] += - rate;
	// Oxygen (0) consumption (rate will be negative), supply nitrite (2) and oxygen (0) concn
	rate = [q->data getRateForReaction: 12 Sub1: s[2][xi][yi] Sub2: s[0][xi][yi] ];
	[self doleOut: (GRate) rate To: (GRate**) r[0] X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
	//r[0][xi][yi] += rate;
	break;

      default: break; // don't do anything for Eps, animals, and other agents not doing substrates
      }
    }
    q = [q pickWin: mobileRoot->mcr Root: mobileRoot Invisibles: 0];
  }
  return self;
}

-update_r_MatricesList {
  // We want to know x & y position and params for our substrate foo at the given concn (we know the concn)
  // Let agents calculate the reaction rate from this and store their answer in the reaction rate matrix r
  
  int k, i, j;
  int xi, yi;
  GCoord xf, yf, radius;
  GRate rate;
  id bac1Index, bac1, bac2Index, bac2;

  // zero the matrices
  for (k=0; k < numSubs; k++)
    [self fillLattice: (GRate**) r[k] WithFloats: 0.0];
  
  bac1Index= [bac1List begin: [self getZone]];
  while (bac1=[bac1Index next]) { // Nitroso: process 1
    
    // get the agents position, note that agents x, y coordinates are GCoord, i.e. floats
    xf  = [bac1 getGX]; // agent coords, f for float
    yf  = [bac1 getGY];
    xi  = (int) (xf / cellSize); // agent coord converted to integer lattice position, that's why i
    yi  = (int) (yf / cellSize);
    // get the agents radius
    radius = [bac1 getRadius];
    
    // Ammonia (1) consumption (rate will be negative), supply ammonia (1) and oxygen (0) concn
    rate = [bac1 getRateForReaction: 1 Sub1: s[1][xi][yi] Sub2: s[0][xi][yi] ];
    [self doleOut: (GRate) rate To: (GRate**) r[1] X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
    //r[1][xi][yi] += rate;
    // Nitrite (2) production (rate will be positive), follows from stoichiometry
    [self doleOut: (GRate) -rate To: (GRate**) r[2] X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
    //r[2][xi][yi] += - rate;
    // Oxygen (0) consumption (rate will be negative), supply ammonia (1) and oxygen (0) concn
    rate = [bac1 getRateForReaction: 2 Sub1: s[1][xi][yi] Sub2: s[0][xi][yi] ];
    [self doleOut: (GRate) rate To: (GRate**) r[0] X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
    //r[0][xi][yi] += rate;
  }
  [bac1Index drop];

  bac2Index= [bac2List begin: [self getZone]];
  while (bac2=[bac2Index next]) { // Nitro: process 2

    // get the agents position, note that agents x, y coordinates are GCoord, i.e. floats
    xf  = [bac2 getGX]; // agent coords, f for float
    yf  = [bac2 getGY];
    xi  = (int) (xf / cellSize); // agent coord converted to integer lattice position, that's why i
    yi  = (int) (yf / cellSize);
    // get the agents radius
    radius = [bac2 getRadius];
    
    // Nitrite (2) consumption (rate will be negative), supply nitrite (2) and oxygen (0) concn
    rate = [bac2 getRateForReaction: 11 Sub1: s[2][xi][yi] Sub2: s[0][xi][yi] ];
    [self doleOut: (GRate) rate To: (GRate**) r[2] X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
    //r[2][xi][yi] += rate;
    // Nitrate (3) production (rate will be positive), follows from stoichiometry
    [self doleOut: (GRate) -rate To: (GRate**) r[3] X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
    //r[3][xi][yi] += - rate;
    // Oxygen (0) consumption (rate will be negative), supply nitrite (2) and oxygen (0) concn
    rate = [bac2 getRateForReaction: 12 Sub1: s[2][xi][yi] Sub2: s[0][xi][yi] ];
    [self doleOut: (GRate) rate To: (GRate**) r[0] X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
    //r[0][xi][yi] += rate;
  }
  [bac2Index drop];

  return self;
}

-update_r_Matrix: (int) k {
  // We want to know x & y position and params for our substrate foo at the given concn (we know the concn)
  // Let agents calculate the reaction rate from this and store their answer in the reaction rate matrix r
  
  int i, j;
  int xi, yi;
  GCoord xf, yf, radius;
  GRate rate, avgSub1, avgSub2;
  id bac1Index, bac1, bac2Index, bac2;

  // zero the matrix
  [self fillLattice: (GRate**) r[k] WithFloats: 0.0];
  
  if (k != 3) { // Nitroso doesn't deal with Nitrate (3)
    bac1Index= [bac1List begin: [self getZone]];
    while (bac1=[bac1Index next]) { // Nitroso: process 1
      
      // get the agents position, note that agents x, y coordinates are GCoord, i.e. floats
      xf  = [bac1 getGX]; // agent coords, f for float
      yf  = [bac1 getGY];
      //xi  = (int) (xf / cellSize); // agent coord converted to integer lattice position, that's why i
      //yi  = (int) (yf / cellSize);
      // get the agents radius
      radius = [bac1 getRadius];
      
      switch (k) {
      case 0: // Oxygen (0) consumption (rate will be negative), supply ammonia (1) and oxygen (0) concn
	avgSub1 = [self avgSub: 1 X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius ];
	avgSub2 = [self avgSub: 0 X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius ];
	rate = [bac1 getRateForReaction: 2 Sub1: avgSub1 Sub2: avgSub2 ];
	//rate = [bac1 getRateForReaction: 2 Sub1: s[1][xi][yi] Sub2: s[0][xi][yi] ];
	break;
      case 1: // Ammonia (1) consumption (rate will be negative), supply ammonia (1) and oxygen (0) concn
	avgSub1 = [self avgSub: 1 X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius ];
	avgSub2 = [self avgSub: 0 X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius ];
	rate = [bac1 getRateForReaction: 1 Sub1: avgSub1 Sub2: avgSub2 ];
	//rate = [bac1 getRateForReaction: 1 Sub1: s[1][xi][yi] Sub2: s[0][xi][yi] ];
	break;
      case 2: // Nitrite (2) production (rate will be positive), follows from stoichiometry
	avgSub1 = [self avgSub: 1 X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius ];
	avgSub2 = [self avgSub: 0 X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius ];
	rate = -1.0 * [bac1 getRateForReaction: 1 Sub1: avgSub1 Sub2: avgSub2 ];
	//rate = -1.0 * [bac1 getRateForReaction: 1 Sub1: s[1][xi][yi] Sub2: s[0][xi][yi] ];
	break;
      default: printLog("Warning, case 3 in update_r_Matrix shouldn't happen\n"); break;
      }
      
      [self doleOut: (GRate) rate To: (GRate**) r[k] X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
      //r[k][xi][yi] += rate;
    }
    [bac1Index drop];
  }
  
  if (k != 1) { // Nitro doesn't deal with Ammonia (1)
    bac2Index= [bac2List begin: [self getZone]];
    while (bac2=[bac2Index next]) { // Nitro: process 2
      
      // get the agents position, note that agents x, y coordinates are GCoord, i.e. floats
      xf  = [bac2 getGX]; // agent coords, f for float
      yf  = [bac2 getGY];
      //xi  = (int) (xf / cellSize); // agent coord converted to integer lattice position, that's why i
      //yi  = (int) (yf / cellSize);
      // get the agents radius
      radius = [bac2 getRadius];
      
      switch (k) {
      case 0: // Oxygen (0) consumption (rate will be negative), supply nitrite (2) and oxygen (0) concn
	avgSub1 = [self avgSub: 2 X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius ];
	avgSub2 = [self avgSub: 0 X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius ];
	rate = [bac2 getRateForReaction: 12 Sub1: avgSub1 Sub2: avgSub2 ];
	//rate = [bac2 getRateForReaction: 12 Sub1: s[2][xi][yi] Sub2: s[0][xi][yi] ];
	break;
      case 2: // Nitrite (2) consumption (rate will be negative), supply nitrite (2) and oxygen (0) concn
	avgSub1 = [self avgSub: 2 X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius ];
	avgSub2 = [self avgSub: 0 X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius ];
	rate = [bac2 getRateForReaction: 11 Sub1: avgSub1 Sub2: avgSub2 ];
	//rate = [bac2 getRateForReaction: 11 Sub1: s[2][xi][yi] Sub2: s[0][xi][yi] ];
	break;
      case 3: // Nitrate (3) production (rate will be positive), follows from stoichiometry
	avgSub1 = [self avgSub: 2 X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius ];
	avgSub2 = [self avgSub: 0 X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius ];
	rate = -1.0 * [bac2 getRateForReaction: 11 Sub1: avgSub1 Sub2: avgSub2 ];
	//rate = -1.0 * [bac2 getRateForReaction: 11 Sub1: s[2][xi][yi] Sub2: s[0][xi][yi] ];
	break;
      default: printLog("Warning, case 1 in update_r_Matrix shouldn't happen\n"); break;
      }
      
      [self doleOut: (GRate) rate To: (GRate**) r[k] X: (GCoord) xf Y: (GCoord) yf Radius: (GCoord) radius];
      //r[k][xi][yi] += rate;
    }
    [bac2Index drop];
  }
  return self;
}

// Courtesy Cristian Picioreanu <C.Picioreanu@STM.TUDelft.NL>
-tridag { // solves a tridiagonal matrix
  // all originally passed args were made ivars
  int i;
  GRate bet;
  
  if (bb[1] == 0.0){
    return self;
  }
  bet = bb[1];
  uu[1] = rr[1]/bet;
  for (i = 2; i <= jj; i++){
    gam[i] = cc[i-1]/bet;
    bet = bb[i]-aa[i]*gam[i];
    if (bet == 0.0) {
      return self;
    }
    uu[i] = (rr[i]-aa[i]*uu[i-1])/bet;
  }
  for (i = jj-1; i >= 1; i--)
    uu[i] = uu[i]-gam[i+1]*uu[i+1];
  
  return self;
}

// Courtesy Cristian Picioreanu <C.Picioreanu@STM.TUDelft.NL>
-tridagPeriodic { // for periodic boundaries, the matrix is not fully tridiagonal and has to be transformed first
  // all originally passed args were made ivars

  int i, i_;
  GRate ratio1, ratio2;
  
  for (i=1; i<=jj; i++) { vv[i]=0.0; hh[i]=0.0; }
  hh[1]= aa[2];      vv[1]= cc[jj-1];
  hh[jj-1]= aa[jj];  vv[jj-1]= cc[jj-1];
  
  for (i=2; i<=jj-1; i++) {
    i_=i-1;
    
    ratio2 = aa[i]/bb[i_];
    
    bb[i] = bb[i] - ratio2*cc[i_];
    rr[i] = rr[i] - ratio2*rr[i_];
    vv[i] = vv[i] - ratio2*vv[i_];
    
    ratio1 = hh[i_]/bb[i_];
    
    hh[i] = hh[i]  - ratio1*cc[i_];
    bb[jj]= bb[jj] - ratio1*vv[i_];
    rr[jj]= rr[jj] - ratio1*rr[i_];
  }

  ratio1= hh[jj-1]/bb[jj-1];
  bb[jj]= bb[jj] - ratio1*vv[jj-1];
  rr[jj]= rr[jj] - ratio1*rr[jj-1];
  
  uu[jj] = rr[jj]/bb[jj];
  uu[jj-1]= (rr[jj-1] - vv[jj-1]*uu[jj]) / bb[jj-1];
  for (i=jj-2; i>=1; i--) uu[i] = (rr[i]-cc[i]*uu[i+1]-vv[i]*uu[jj])/bb[i];
  
  return self;
}

-recalcParams {
  int i;
  // We need these params for ADI
  // They are diffusion-timestep (dt) dependent, hence, they have to be recalculated only when dt changes
  // We normalize the length of the world in x and y, and the subConcn
  // We don't normalize the reaction rate, not necessary and not easy to do, but choose units in order to
  // have reaction rates close to unity

  // The reaction rate coefficient must be normalized by extConcn
  // Bacilli think amounts, we think concn, hence we have to divide by volume
  for (i=0; i < numSubs; i++) {
    Ar[i] = dtmin / (2.0 * cellVol * extConcn[i]);
  }

  return self;
}

// Courtesy Cristian Picioreanu <C.Picioreanu@STM.TUDelft.NL>
-adi: (int) k { // call adi for sub k
  // we consider the center of the lattice elements, not the nodes
  int i, j;
  
  maxDelta = 0.0;

  // ---- Step 1 ----- Diffusion only in Y direction --------------
  for (j = 1; j < cellsX-1; j++) { // terms in X direction are constant
    jj = 0;	// counts internal lattice points
    for (i = 1; i < cellsY-1; i++) { // --- Collecting terms for Y direction ---
      if (cs[i][j]!=0){ // don't calculate in the bulk liquid
	jj++;
	// calculate the right hand side of diffusion-reaction (directions orthogonal to diffusion direction)
	rr[jj] = s[k][i][j+1] + (d[k][i][j]-2.0)*s[k][i][j] + s[k][i][j-1] + Ar[k]*d[k][i][j]*r[k][i][j];
	
	if (cs[i-1][j] == 0){
	  rr[jj] += s[k][i-1][j];
	  aa[jj] = 0.0;
	}else{
	  aa[jj] = -1.0;
	}
	
	if (cs[i+1][j] == 0){
	  rr[jj] += s[k][i+1][j];
	  cc[jj] = 0.0;
	}else{
	  cc[jj] = -1.0;
	}

	bb[jj] = 2.0 + d[k][i][j];
      }
    }

    if (jj > 0) [self tridag];
		
    jj = 0;
    
    for (i = 1; i < cellsY-1; i++){
      if (cs[i][j]!=0){
	jj++;
	tmp[i][j] = (uu[jj] >= 0.0 ? uu[jj] : 0.0);
      }else{
	tmp[i][j] = 1.0;
      }
    }
  }
  
  // ---- Step 2 ----- Diffusion only in X direction --------------
  for (i = 1; i < cellsY-1; i++) {
    jj = 0;
    for (j = 1; j < cellsX-1; j++) {
      if (cs[i][j]!=0){
	jj++;
	rr[jj] = tmp[i+1][j] + (d[k][i][j]-2.0)*tmp[i][j] + tmp[i-1][j] + Ar[k]*d[k][i][j]*r[k][i][j];

	if (cs[i][j-1] == 0){
	  rr[jj] += tmp[i][j-1];
	  aa[jj] = 0.0;
	}else{
	  aa[jj] = -1.0;
	}

	if (cs[i][j+1] == 0){
	  rr[jj] += tmp[i][j+1];
	  cc[jj] = 0.0;
	}else{
	  cc[jj] = -1.0;
	}
	
	bb[jj] = 2.0 + d[k][i][j];
      }
    }
    
    if (jj > 0) [self tridag];
    
    jj = 0;

    for (j = 1; j < cellsX-1; j++){
      if (cs[i][j]!=0){
	jj++;
	//if (uu[jj] <= 0) {printLogD("solution is %g\t", uu[jj]); printLog("in %d, %d\n", i, j);}
	// Here we update the s matrix, before that, store the differences away for printing
	//delta[i][j] = s[k][i][j] - uu[jj];
	// Are we converging?
	if (fabs(uu[jj]-s[k][i][j]) > maxDelta) maxDelta = fabs(uu[jj]-s[k][i][j]);
	s[k][i][j] = (uu[jj] >= 0.0 ? uu[jj] : 0.0);
      }else{
	s[k][i][j] = 1.0;
      }
    }
  }

  return self;
}

// Courtesy Cristian Picioreanu <C.Picioreanu@STM.TUDelft.NL>
-adiPlate: (int) k { // call adiPlate for sub k
  // we consider the center of the lattice elements, not the nodes

  int top;
  int caz;
  int i, xi, yi;
  int xm, xp, ym, yp;
  
  maxDelta = 0.0;
  top = front + boundaryLayer + 1;

  for (yi = 0; yi < cellsY; yi++) {
    jj = 0;
    for (i = 0; i < top+1; i++) { aa[i]=0.0; bb[i]=0.0; cc[i]=0.0; rr[i]=0.0; };
    for (xi = 0; xi < top; xi++) {
      
      if (yi==0) ym=cellsY-1; else ym=yi-1;
      if (yi==cellsY-1) yp=0; else yp=yi+1;
      xm=xi-1;
      xp=xi+1;

      if (cs[xi][yi]==1) {
	jj++;
	
	caz = cs[xi][ym] + 4*cs[xi][yp];
	switch (caz) {
	case 0: case 1: case 4: case 5: // no carrier cases
	  rr[jj] = s[k][xi][yp] + (d[k][xi][yi]-2.0)*s[k][xi][yi] +   s[k][xi][ym] + Ar[k]*d[k][xi][yi]*r[k][xi][yi];
	  break;
	case 8: case 9: // carrier in yp
	  rr[jj] =                (d[k][xi][yi]-2.0)*s[k][xi][yi] + 2*s[k][xi][ym] + Ar[k]*d[k][xi][yi]*r[k][xi][yi];
	  break;
	case 2: case 6: // carrier in ym
	  rr[jj] =                (d[k][xi][yi]-2.0)*s[k][xi][yi] + 2*s[k][xi][yp] + Ar[k]*d[k][xi][yi]*r[k][xi][yi];
	  break;
	case 10:        // carrier both in ym and yp, gives rise to a singularity in tridag, avoid this case
	  rr[jj] =                (d[k][xi][yi]-2.0)*s[k][xi][yi]                  + Ar[k]*d[k][xi][yi]*r[k][xi][yi];
	  break;
	}
	
	caz = cs[xm][yi] + 4*cs[xp][yi];
	switch (caz) {                                                                             // xp      xm
	case 5:	 aa[jj]=-1.0; cc[jj]=-1.0;                                                  break; // bio     bio
	case 6:	 aa[jj]= 0.0; cc[jj]=-2.0;                                                  break; // carrier bio
	case 9:	 aa[jj]=-2.0; cc[jj]= 0.0;                                                  break; // bio     carrier
	case 4:  aa[jj]= 0.0; cc[jj]=-1.0; rr[jj] = rr[jj] +   s[k][xm][yi];                break; // bulk    bio
	case 1:	 aa[jj]=-1.0; cc[jj]= 0.0; rr[jj] = rr[jj] +   s[k][xp][yi];                break; // bio     bulk
	case 0:	 aa[jj]= 0.0; cc[jj]= 0.0; rr[jj] = rr[jj] +   s[k][xm][yi] + s[k][xp][yi]; break; // bulk    bulk
	case 8:	 aa[jj]= 0.0; cc[jj]= 0.0; rr[jj] = rr[jj] + 2*s[k][xm][yi];                break; // bulk    carrier
	case 2:	 aa[jj]= 0.0; cc[jj]= 0.0; rr[jj] = rr[jj] + 2*s[k][xp][yi];                break; // carrier bulk
	case 10: aa[jj]= 0.0; cc[jj]= 0.0;                                                  break; // carrier carrier
	}
	
	bb[jj] = 2.0 + d[k][xi][yi];
      }
    }
    
    if (jj > 0) [self tridag];
    
    jj = 0;
    for (xi = 0; xi < cellsX; xi++) {
      if (cs[xi][yi]==1) {
	jj++;
	tmp[xi][yi] = (uu[jj]>=0.0 ? uu[jj] : 0.0);
      }else{
	if (cs[xi][yi]==0) tmp[xi][yi] = 1.0;
	if (cs[xi][yi]==2) tmp[xi][yi] = 0.0;
      }
    }
  }
  
  for (xi = 0; xi < top; xi++) {
    for (i = 0; i < cellsY+1; i++) { aa[i]=0.0; bb[i]=0.0; cc[i]=0.0; rr[i]=0.0; };
    jj = 0;
    for (yi = 0; yi < cellsY; yi++) {
      
      if (yi==0) ym=cellsY-1; else ym=yi-1;
      if (yi==cellsY-1) yp=0; else yp=yi+1;
      xm=xi-1;
      xp=xi+1;

      if (cs[xi][yi]==1){
	jj++;
	
	caz = cs[xm][yi] + 4*cs[xp][yi];
	switch (caz){
	case 0: case 1: case 4: case 5:
	  rr[jj] = tmp[xp][yi] + (d[k][xi][yi]-2.0)*tmp[xi][yi] +   tmp[xm][yi] + Ar[k]*d[k][xi][yi]*r[k][xi][yi];
	  break;
	case 8: case 9:
	  rr[jj] =               (d[k][xi][yi]-2.0)*tmp[xi][yi] + 2*tmp[xm][yi] + Ar[k]*d[k][xi][yi]*r[k][xi][yi];
	  break;
	case 2: case 6:
	  rr[jj] =               (d[k][xi][yi]-2.0)*tmp[xi][yi] + 2*tmp[xp][yi] + Ar[k]*d[k][xi][yi]*r[k][xi][yi];
	  break;
	case 10:
	  rr[jj] =               (d[k][xi][yi]-2.0)*tmp[xi][yi]                 + Ar[k]*d[k][xi][yi]*r[k][xi][yi];
	  break;
	}
	
	caz = cs[xi][ym] + 4*cs[xi][yp];
	switch (caz){
	case 5:	 aa[jj]=-1.0; cc[jj]=-1.0;                                                break;
	case 6:	 aa[jj]= 0.0; cc[jj]=-2.0;                                                break;
	case 9:	 aa[jj]=-2.0; cc[jj]= 0.0;                                                break;
	case 4:  aa[jj]= 0.0; cc[jj]=-1.0; rr[jj] = rr[jj] +   tmp[xi][ym];               break;
	case 1:	 aa[jj]=-1.0; cc[jj]= 0.0; rr[jj] = rr[jj] +   tmp[xi][yp];               break;
	case 0:	 aa[jj]= 0.0; cc[jj]= 0.0; rr[jj] = rr[jj] +   tmp[xi][ym] + tmp[xi][yp]; break;
	case 8:	 aa[jj]= 0.0; cc[jj]= 0.0; rr[jj] = rr[jj] + 2*tmp[xi][ym];               break;
	case 2:	 aa[jj]= 0.0; cc[jj]= 0.0; rr[jj] = rr[jj] + 2*tmp[xi][yp];               break;
	case 10: aa[jj]= 0.0; cc[jj]= 0.0;                                                break;
	}
	
	bb[jj] = 2.0 + d[k][xi][yi];
      }
    }
    
    if (jj > 0) [self tridagPeriodic];
    
    jj = 0;
    for (yi = 0; yi < cellsY; yi++) {
      if (cs[xi][yi]==1) {
	jj++;
	if (fabs(uu[jj]-s[k][xi][yi]) > maxDelta) maxDelta = fabs(uu[jj]-s[k][xi][yi]);
	s[k][xi][yi] = (uu[jj]>=0.0 ? uu[jj] : 0.0);
      }else{
	if (cs[xi][yi]==0) s[k][xi][yi] = 1.0;
	if (cs[xi][yi]==2) s[k][xi][yi] = 0.0;
      }
    }
  }
  return self;  
}

-stepResources {
  // This is the "main" routine of diffusion,
  // scheduled to repeat every plantcycle by the model swarm via landroot.
  // We don't remove any substrates but we consider reaction rates
  // We have to do some bookkeeping for the sake of our landroot!

  BOOL found = FALSE;
  Quad* q;
  Quad* mobileRoot = [[geckoControl getModelSwarm] getMobileRoot];

  GRate minPorosity;

  int i, j, p;
  int trues;

  char filename[256];
  FILE *file;
  timeval_t now = [geckoControl getCurrentTime];
  id bac1Index, bac1, bac2Index, bac2, epsIndex, eps;

  countDiff = 0;
  stable  = TRUE;  // unless proven otherwise by the diffusion engine
  relaxed = FALSE; // set to false before we start diffusion, if finished, set to true again
  for (i=0; i < numSubs; i++) relaxedSub[i] = FALSE;

  // before everything else, tell critters to shove if biofilm is overcrowded
  q = [mobileRoot pickWin: mobileRoot->mcr Root: mobileRoot Invisibles: 0];
  while (!found) {
    if ([q->data getProcess] > 0) { // if not Eps
      found = TRUE;
      haveEps = [q->data haveEps]; // convenient place to ask, should be early enough
#ifdef EXTRALOCO
      if ([self calcPorosity] < 0.26) minPorosity = [q->data doTheLocomotion];
#else EXTRALOCO
      minPorosity = [self calcPorosity]; // to disregard crowding
#endif EXTRALOCO
    } else {
      q = [q pickWin: mobileRoot->mcr Root: mobileRoot Invisibles: 0];
    }
  }

  [self update_x_Matrices]; // first update biomass, also check for agents outside landscape
  [self update_c_Matrix];   // it's easy to update c by looking at xtot
  [self update_cs_Matrix];  // cs depends on c, also check for front over the top
  [self update_d_Matrices]; // d depends on xbac

  if ([geckoControl getWantToStop]) {
    [geckoControl setWantToStop: FALSE]; // if we would forget this, the next run would be stopped immediately
    [geckoControl stopRunning];
    return self;
  }

  // Fill lists of agents for faster access
  q = [mobileRoot pickWin: mobileRoot->mcr Root: mobileRoot Invisibles: 0];
  while (q) {
    p = [q->data getProcess];
    switch (p) {
    case -1:  [epsList addLast: q->data]; break;
    case  1: [bac1List addLast: q->data]; break;
    case  2: [bac2List addLast: q->data]; break;
    default: printLog("Warning, undefined case in List shouldn't happen\n");
    }
    q = [q pickWin: mobileRoot->mcr Root: mobileRoot Invisibles: 0];
  }
  
  do {

    //[self update_r_Matrices]; // necessary every time the subConcn changes
    //[self update_r_MatricesList]; // necessary every time the subConcn changes

    //if (countDiff % 50 == 0) printDiff("%d\t", [geckoControl getCurrentTime]); // prints the start of the max[i] line

    for (i=0; i < numSubs; i++) {
      // call the diffusion engine for sub i, but only if conversion for i has not been reached already
      if (!relaxedSub[i]) {
	[self update_r_Matrix: i]; // necessary every time the subConcn changes
	if (plate) [self adiPlate: i]; else [self adi: i];
	max[i] = maxDelta/dtfix;
	if (countDiff >= 100) {
	  if ((max[i] - maxlast[i]) > 0) { // do we have an upward trend?
	    stable = FALSE;
	  }
	}
	if (countDiff % 50 == 0) maxlast[i] = max[i];
	if (max[i] < 1.0)  relaxedSub[i] = TRUE;
      }
      //if (countDiff % 50 == 0) { printDiffD("%06.4f\t", max[i]);} // prints the max[i] line
    }

    //if (countDiff % 50 == 0) printDiff("\n"); // prints the end of the max[i] line
    
    countDiff++; totDiffSteps++;

    trues = 0;
    for (i=0; i < numSubs; i++) if (relaxedSub[i] == TRUE) trues++;
    if (trues == numSubs) relaxed = TRUE;

    // check whether we should make our diffusion timestep smaller
    if (!stable) {
      dtmin = dtmin * 0.9;
      stable = TRUE;
      printLogD("Diffusion time step decreased to %g ", dtmin); printLog("at %d\n", [geckoControl getCurrentTime]);
      countDiff = 0;
      for (i=0; i < numSubs; i++) relaxedSub[i] = FALSE;
      lastDtminChange = [geckoControl getCurrentTime];
      [self recalcParams];      // recalc diffusion time step (dt) dependent coefficients
      [self update_d_Matrices]; // depend on dt also
    }
    
    if (countDiff > 8000) {
      stable = FALSE;
    }
    
  } while (!relaxed);

  // occasionally check whether we should make our diffusion timestep larger
  // only do so if our last instability prompted decrease of dtmin is "long" ago
  if ( (countDiff > 1000) && (([geckoControl getCurrentTime] - lastDtminChange) > 10) ) {
    dtmin = dtmin * 1.14;
    lastDtminChange = [geckoControl getCurrentTime];
    printLogD("Diffusion time step increased to %g ", dtmin); printLog("at %d\n", [geckoControl getCurrentTime]);
    [self recalcParams];      // recalc diffusion time step (dt) dependent coefficients
  }

  printDiff("%d\t", [geckoControl getCurrentTime]);
  printDiff("%d\n", countDiff);

  startres = 0.0;
  for (i=0; i < cellsX; i++) {
    for (j=0; j < cellsY; j++) {
      startres += s[2][i][j];
    }
  }
  reservoir = startres;

  // Printing and saving rngState is done here, _before_ the growth step, and _after_ the substrate
  // field is relaxed. At this time, the bacteria are frozen, the rng is not used, and the substrates are relaxed.
  if (now % [geckoControl getStatCycle] == 0) {
  // noise run:
  //if ( ((now >= 975) && (now <= 1025)) || ((now >= 9975) && (now <= 10025)) || ((now >= 29975) && (now <= 30025)) ) {

#ifdef VERBOSE
    [self printv]; // prints substrate and reaction lattices, can be changed to print more
#endif VERBOSE
    //[self printLatticeHet];  // print SD of subConcn heterogeneity over total lattice
    [self printSubConcnHet]; // this prints the heterogeneity over the cells only
    
    i = 1;
    q = [mobileRoot pickWin: mobileRoot->mcr Root: mobileRoot Invisibles: 0];
    while (q) {
      if (i == 1 && [q->data getProcess] > 0) {
	[q->data printFed];
	i++;
      }
      [q->data printSpheresMatlab];
      q = [q pickWin: mobileRoot->mcr Root: mobileRoot Invisibles: 0];
    }

    // If there is an in file, we have read it, so we should also read the rng-state
    sprintf(filename, "bac_%03ld_%06ld.in", [geckoControl getNumRun], [geckoControl getCurrentTime]);
    file = fopen(filename, "r");
    if (file) {
      fclose(file);
      [randSource readRngState];
    }	
    if ( ([geckoControl getRestartFlag]) && (now == [geckoControl getStartTime]) && \
	 ([geckoControl getNumRun] == [geckoControl getStartRun]) )  {
      [randSource readRngState];
    } else {
      [randSource saveRngState];
    }
  }

  [bac1List removeAll]; [bac2List removeAll]; [epsList removeAll];

  return self;
}

// the following exist for the convenience of the Params scheme--not for calling directly.
-setNumSubs: (int) i {numSubs =  i; return self;}
-setNumSpecies: (int) i {numSpecies =  i; return self;}
-setCellSize: (GCoord) xi {cellSize = xi; return self;}
-setCellsX: (int) i {cellsX =  i; return self;}
-setCellsY: (int) i {cellsY =  i; return self;}
-(int) getCellsX {return cellsX;}
-(int) getCellsY {return cellsY;}
-setPlate: (int) p {plate = p; return self;}
-setBoundaryLayer: (int) b {boundaryLayer = (int) rint(b / cellSize); return self;}
-setCarrierLayer: (int) cl {carrierLayer = cl; return self;}
-setSliceThickness: (GCoord) st {sliceThickness = st; return self;}
-setDiffC0: (GRate) xi {diffC0 = xi * [[geckoControl getModelSwarm] getStepSize]; return self;}
-setDiffC1: (GRate) xi {diffC1 = xi * [[geckoControl getModelSwarm] getStepSize]; return self;}
-setDiffC2: (GRate) xi {diffC2 = xi * [[geckoControl getModelSwarm] getStepSize]; return self;}
-setDiffC3: (GRate) xi {diffC3 = xi * [[geckoControl getModelSwarm] getStepSize]; return self;}
-setExtConcn0: (GRate) xi {extConcn0 =  xi; return self;}
-setExtConcn1: (GRate) xi {extConcn1 =  xi; return self;}
-setExtConcn2: (GRate) xi {extConcn2 =  xi; return self;}
-setExtConcn3: (GRate) xi {extConcn3 =  xi; return self;}
-setNumColors: (int) i {numColors =  i; return self;}
-setHighColorDensity: (GRate) xi {highColorDensity = xi; return self;}
-setLowColorDensity: (GRate) xi {lowColorDensity =  xi; return self;}

// these are used in QuadDisplay
-(GCoord) getMcrLXCellX: (int) xi Y: (int) yi {return (mcr->lx + (xi * cellSize));}
-(GCoord) getMcrLYCellX: (int) xi Y: (int) yi {return (mcr->ly + (yi * cellSize));}
-(GCoord) getMcrUXCellX: (int) xi Y: (int) yi {return (mcr->lx + ((xi + 1) * cellSize));}
-(GCoord) getMcrUYCellX: (int) xi Y: (int) yi {return (mcr->ly + ((yi + 1) * cellSize));}

// printing routines at the end of the file
-printv {
  int i, j, k;
  GRate d; 
  char filename[256];
  FILE *file;
  timeval_t now = [geckoControl getCurrentTime];
  
  // there are numSubs lattices for subConcn and reaction rate

  for (k=0; k < numSubs; k++) {
    // print the subConcn lattices
    sprintf(filename, "%s_%03ld_s%d_%06ld.ml", [geckoControl getFileStem], [geckoControl getNumRun], k, now);
    file = fopen(filename, "w"); // overwrite existing file
    for (j=0; j < cellsY; j++) {
      for (i=0; i < cellsX; i++) {
	//fprintf(file, "%8.6f ", s[k][i][j]); // easier reading, but many times larger files
	fprintf(file, "%g ", s[k][i][j]);
      }
      fprintf(file, "\n");
    }
    fclose(file);
    
//      // print the reaction rate lattices
//      sprintf(filename, "%s_%03ld_r%d_%06ld.ml", [geckoControl getFileStem], [geckoControl getNumRun], k, now);
//      file = fopen(filename, "w"); // overwrite existing file
//      for (j=0; j < cellsY; j++) {
//        for (i=0; i < cellsX; i++) {
//  	fprintf(file, "%g ", r[k][i][j]);
//        }
//        fprintf(file, "\n");
//      }
//      fclose(file);

  }

  // there are numSpecies biomass and growth rate lattices

  for (k=0; k < numSpecies; k++) {
    // print the biomass lattices
    sprintf(filename, "%s_%03ld_x%d_%06ld.ml", [geckoControl getFileStem], [geckoControl getNumRun], k, now);
    file = fopen(filename, "w"); // overwrite existing file
    for (j=0; j < cellsY; j++) {
      for (i=0; i < cellsX; i++) {
	fprintf(file, "%g ", x[k][i][j]);
      }
      fprintf(file, "\n");
    }
    fclose(file);
//      // print the growth rate lattices
//      sprintf(filename, "%s_%03ld_g%d_%06ld.ml", [geckoControl getFileStem], [geckoControl getNumRun], k, now);
//      file = fopen(filename, "w"); // overwrite existing file
//      for (j=0; j < cellsY; j++) {
//        for (i=0; i < cellsX; i++) {
//  	fprintf(file, "%g ", g[k][i][j]);
//        }
//        fprintf(file, "\n");
//      }
//      fclose(file);
  }

//    // print the total biomass lattice
//    sprintf(filename, "%s_%03ld_xt_%06ld.ml", [geckoControl getFileStem], [geckoControl getNumRun], now);
//    file = fopen(filename, "w"); // overwrite existing file
//    for (j=0; j < cellsY; j++) {
//      for (i=0; i < cellsX; i++) {
//        fprintf(file, "%g ", xtot[i][j]);
//      }
//      fprintf(file, "\n");
//    }
//    fclose(file);

//    // print the total cellular biomass lattice, only if we have eps
//    if (haveEps) {
//      sprintf(filename, "%s_%03ld_xb_%06ld.ml", [geckoControl getFileStem], [geckoControl getNumRun], now);
//      file = fopen(filename, "w"); // overwrite existing file
//      for (j=0; j < cellsY; j++) {
//        for (i=0; i < cellsX; i++) {
//  	fprintf(file, "%g ", xbac[i][j]);
//        }
//        fprintf(file, "\n");
//      }
//      fclose(file);
    
//      // print the eps lattice, only if we have eps
//      sprintf(filename, "%s_%03ld_xe_%06ld.ml", [geckoControl getFileStem], [geckoControl getNumRun], now);
//      file = fopen(filename, "w"); // overwrite existing file
//      for (j=0; j < cellsY; j++) {
//        for (i=0; i < cellsX; i++) {
//  	fprintf(file, "%g ", xeps[i][j]);
//        }
//        fprintf(file, "\n");
//      }
//      fclose(file);
//    }
  
//    // print the porosity lattice
//    sprintf(filename, "%s_%03ld_p_%06ld.ml", [geckoControl getFileStem], [geckoControl getNumRun], now);
//    file = fopen(filename, "w"); // overwrite existing file
//    for (j=0; j < cellsY; j++) {
//      for (i=0; i < cellsX; i++) {
//        fprintf(file, "%g ", por[i][j]);
//      }
//      fprintf(file, "\n");
//    }
//    fclose(file);

  return self;
}

-printColorThresholds {
  int i;
  printLog("Colors:\n");
  for (i=0; i < numColors; i++) {
    printLogD("\tdensity >= %lf", colorDensity[i]);
    printLog("\t%d\n", colorIds[i]);
  }
  return self;
}

-printLatticeHet { // print SD of subConcn heterogeneity over total lattice
  int i, j, k;
  int num; // number of lattice cells
  ResQuant mean; // average subConcn in lattice
  ResQuant sq; // sum of squared deviations
  ResQuant stdDev; // Standard Deviation of SubConcn: not a sample SD, hence SD(n) not SD(n-1)
  ResQuant sub; // subConcn
  timeval_t now = [geckoControl getCurrentTime];

  for (k=0; k < numSubs; k++) {
    mean = 0.0;
    num  = 0;
    sq = 0.0;

    for (i=0; i < cellsX; i++) {
      for (j=0; j < cellsY; j++) {
	// count only lattice cells without carrier
	if (c[i][j] != 2) {
	  mean += s[k][i][j];
	  num++;
	}
      }
    }
    
    mean = mean / num;
    
    for (i=0; i < cellsX; i++) {
      for (j=0; j < cellsY; j++) {	
	// count only lattice cells without carrier
	if (c[i][j] != 2) {
	  sub = s[k][i][j];
	  sq += pow((sub - mean), 2.0);
	}
      }
    }

    stdDev = sqrt(sq / num);
    printBac("%d\t%d\t", [geckoControl getNumRun], now);
    printBac("%d\t", k);
    printBacD("%g\t", mean);
    printBacD("%g\t", stdDev);
    printBac("%d\n", num);
  }
  return self;
}

-printSubConcnHet { // print SD of subConcn heterogeneity over all bacs
  int i, j, k, l;
  int num; // number of lattice cells with bacteria of that type
  ResQuant mean; // average subConcn in lattice
  ResQuant sq; // sum of squared deviations
  ResQuant stdDev; // Standard Deviation of SubConcn: not a sample SD, hence SD(n) not SD(n-1)
  ResQuant sub; // subConcn
  timeval_t now = [geckoControl getCurrentTime];

  for (k=0; k < numSubs; k++) {
    for (l=0; l < numSpecies; l++) {
      mean = 0.0;
      num  = 0;
      sq = 0.0;

      for (i=0; i < cellsX; i++) {
	for (j=0; j < cellsY; j++) {
	  // count only lattice cells with bacteria of type l and no carrier
	  if ( (x[l][i][j] > 0) && (c[i][j] != 2) ) {
	    mean += s[k][i][j];
	    num++;
	  }
	}
      }
      
      mean = mean / num;

      for (i=0; i < cellsX; i++) {
	for (j=0; j < cellsY; j++) {	
	  // count only lattice cells with bacteria of type l and no carrier
	  if ( (x[l][i][j] > 0) && (c[i][j] != 2) ) {
	    sub = s[k][i][j];
	    sq += pow((sub - mean), 2.0);
	  }
	}
      }

      stdDev = sqrt(sq / num);
      printBac("%d\t%d\t", [geckoControl getNumRun], now);
      printBac("%d\t", k);
      printBac("%d\t", l);
      printBacD("%g\t", mean);
      printBacD("%g\t", stdDev);
      printBac("%d\n", num);
    }
  }
  return self;
}

-debug {
  int i, j, k;
  GRate d; 
  char filename[256];
  FILE *file;
  timeval_t now = [geckoControl getCurrentTime];

  printDiff("Time %d\t", now);
  printDiff("countDiff %d\n", countDiff);
  for (i=0; i < numSubs; i++) {
    printDiff("max[%d] ", i); printDiffD("%g\t", max[i]);  
  }
  printDiff("\n");

  // there are numSubs lattices for subConcn and reaction rate

  for (k=0; k < numSubs; k++) {
    // print the subConcn lattices
    sprintf(filename, "%s_s%d_%06ld_%d.ml", [geckoControl getFileStem], k, now, countDiff);
    file = fopen(filename, "w"); // overwrite existing file
    for (j=0; j < cellsY; j++) {
      for (i=0; i < cellsX; i++) {
	fprintf(file, "%8.6f ", s[k][i][j]);
      }
      fprintf(file, "\n");
    }
    fclose(file);
    
    // print the reaction rate lattices
    sprintf(filename, "%s_r%d_%06ld_%d.ml", [geckoControl getFileStem], k, now, countDiff);
    file = fopen(filename, "w"); // overwrite existing file
    for (j=0; j < cellsY; j++) {
      for (i=0; i < cellsX; i++) {
	fprintf(file, "%g ", r[k][i][j]);
      }
      fprintf(file, "\n");
    }
    fclose(file);

  }

  // there are numSpecies biomass lattices

  for (k=0; k < numSpecies; k++) {
    // print the biomass lattices
    sprintf(filename, "%s_x%d_%06ld_%d.ml", [geckoControl getFileStem], k, now, countDiff);
    file = fopen(filename, "w"); // overwrite existing file
    for (j=0; j < cellsY; j++) {
      for (i=0; i < cellsX; i++) {
	fprintf(file, "%g ", x[k][i][j]);
      }
      fprintf(file, "\n");
    }
    fclose(file);
  }

  // print the total biomass lattices
  sprintf(filename, "%s_xt_%06ld_%d.ml", [geckoControl getFileStem], now, countDiff);
  file = fopen(filename, "w"); // overwrite existing file
  for (j=0; j < cellsY; j++) {
    for (i=0; i < cellsX; i++) {
      fprintf(file, "%g ", xtot[i][j]);
    }
    fprintf(file, "\n");
  }
  fclose(file);

  return self;
}

@end
