#import "OtherDisplay.h"
#import "Stat.h"
#import "GeckoControlSwarm.h"
#import <activity.h>

// TraceGraph: a graph that updates its traces when stepped.
@implementation TraceGraph

int TracesPerChunk = 30;// have to re-allocate if exceed...not implemented

-init
{
    int arraysize = TracesPerChunk * sizeof(id);
    
    maxTraces = TracesPerChunk;
    numlines  = numInts = numDoubles = 0;
    census     = (unsigned**) [globalZone alloc: arraysize]; 
    values     = (double**) [globalZone alloc: arraysize]; 
    labels     = (char**) [globalZone alloc: arraysize];
    lines      = (GraphElement**) [globalZone alloc: arraysize];
    graph      = [Graph create: globalZone];
    wintitle   = "Population";
    yaxislabel = "";
    log        = 0;
    lastStep   = -1;
    displayFlag = 0;
    return self;
}

+create
{
    id result;
    result = [TraceGraph createBegin: globalZone];
    [result init];
    return result;					
}

+createFat: (int) ntraces
{
    id result;
    int save = TracesPerChunk;
    TracesPerChunk = ntraces;
    result = [TraceGraph createBegin: globalZone];
    [result init];
    TracesPerChunk = save;
    return result;					
}

-track: (unsigned*) censusvar Key: (char*) label Color: (char*) colorname
    // call track for each trace you want, then createEnd
{
    if (numlines == maxTraces) {
        geckoError(InvalidCombination, 
            "TraceGraph:  max %d traces implemented.\n", maxTraces);
        return self; 
    }

    census[numInts] = censusvar;
    labels[numlines] = label;
    lines[numlines]  = [graph createElement];
    [lines[numlines] setColor: colorname];
    [lines[numlines] setLabel: label];
    numlines++;
    numInts++;
    
    return self;        
}
-trackDouble: (double*) valuevar Key: (char*) label Color: (char*) colorname
    // call track for each trace you want, then createEnd
{
    if (numlines == maxTraces) {
        geckoError(InvalidCombination, 
            "TraceGraph:  max %d traces implemented.\n", maxTraces);
        return self; 
    }

    values[numDoubles] = valuevar;
    labels[numlines] = label;
    lines[numlines]  = [graph createElement];
    [lines[numlines] setColor: colorname];
    [lines[numlines] setLabel: label];
    numlines++;
    numDoubles++;
    
    return self;        
}

-setLabel: (int) i To: (char*) label
{
    [lines[i] setLabel: label];
    return self;
}

-clear
{
    int i;
    for (i=0; i<numlines; i++) 
        [lines[i] resetData];
    return self;
}

-setWinTitle: (char*) label
    // optional
{
    wintitle   = label;
    return self;
}

-setYAxisLabel: (char*) label
    // optional
{
    yaxislabel = label;
    return self;
}

-logFile: (char*) filename
  // if you set a log file, a record is appended at each step
{
  if ([geckoControl getRestartFlag]) {
    if (!(log = fopen(filename, "a"))){ // append if restarted run or next run
      geckoError(InvalidCombination,
		 "Error opening log file '%s'.\n", filename);
      return self;
    }
  } else {
    if (!(log = fopen(filename, "w"))){ // overwrite only if first run
      geckoError(InvalidCombination,
		 "Error opening log file '%s'.\n", filename);
      return self;
    }
  }
  return self;
}

-createEnd 
{
    if (numlines == 0) {
        geckoWarning(
            "TraceGraph not initialized properly--no lines.\n", " ");
    }
    
    [graph setTitle: wintitle];
    [graph setAxisLabelsX: "time" Y: yaxislabel];
    if (displayFlag)
        [graph pack];

    return self;
}

-display 
{
    displayFlag = 1;
    [graph pack];
    return self;
}

-logStep
{
    int i;
    
    if (numlines < 1)
        return self;
    
    fprintf(log, "%ld ", [geckoControl getCurrentTime]);
    for (i=0; i < numInts; i++) {
        fprintf(log, "%d ", (*census[i]));
    }
    for (i=0; i < numDoubles; i++) {
        fprintf(log, "%f ", (*values[i]));
    }
    fprintf(log, "\n");
    fflush(log);
    
    return self;
}

-step 
    // add a new point, (currentTime, curValue), to each trace.
    // vfb, 970630, changed to draw 0-line, as seasons looked ugly.
{
    int i;
    double d;
    if ([geckoControl getCurrentTime] == lastStep)
        return self;
    
    lastStep = [geckoControl getCurrentTime];

    for (i=0; i < numInts; i++) {
        d = *census[i];
        [lines[i] addX: [geckoControl getCurrentTime] Y: d];
    }
    for (i=0; i < numDoubles; i++) {
        d = *values[i];
        [lines[i] addX: [geckoControl getCurrentTime] Y: d];
    }
    [self logStep];
    
    return self;
}

@end

// TangleGraph: a graph that updates its traces when stepped.
@implementation TangleGraph

-init
{
    int arraysize = TracesPerChunk * sizeof(id);
    
    [super init];
    numXDoubles = 0;
    valuesX     = (double**) [globalZone alloc: arraysize]; 
    return self;
}

+create
{
    id result;
    result = [TangleGraph createBegin: globalZone];
    [result init];
    return result;					
}

-createEnd 
{
    if (numlines == 0) {
        geckoError(InvalidCombination, 
            "TangleGraph not initialized properly--no lines.\n");
        return self;
    }
    
    [graph setTitle: wintitle];
    [graph setAxisLabelsX: xaxislabel Y: yaxislabel];
    if (displayFlag)
        [graph pack];

    return self;
}

-trackDoubleX: (double*) indepvar Y: (double*) depvar
          Key: (char*) label Color: (char*) colorname
{
    [super trackDouble: depvar Key: label Color: colorname];
    valuesX[numXDoubles] = indepvar;
    numXDoubles++;
    return self;        
}

-setXAxisLabel: (char*) label
    // optional
{
    xaxislabel = label;
    return self;
}

-logStep
{
    int i;
    
    if (numlines < 1)
        return self;
    
    for (i=0; i < numlines; i++) {
        fprintf(log, "%f %f ", (*valuesX[i]), (*values[i]));
    }
    fprintf(log, "\n");
    fflush(log);
    
    return self;
}

-step 
    // add a new point to each trace.
{
    int i;
    double d, x;
    if ([geckoControl getCurrentTime] == lastStep)
        return self;
    
    lastStep = [geckoControl getCurrentTime];

    for (i=0; i < numlines; i++) {
        x = *valuesX[i];
        d = *values[i];
        [lines[i] addX: x Y: d];
    }
    [self logStep];
    
    return self;
}

@end

/*  vfb, 980327, commented out due to const char* const * labels in
    the latest Histogram class...irksome, and I'm not using Fuzzies.

@implementation FuzzyGraph

-init
{
    int arraysize = TracesPerChunk * sizeof(id);
    
    numlines   = 0;
    data       = (unsigned**) [globalZone alloc: arraysize]; 
    onehundred = (unsigned**) [globalZone alloc: arraysize]; 
    labels     = (const char* const *)   [globalZone alloc: arraysize];
    colors     = (char**)   [globalZone alloc: arraysize];
    toplot     = (double*)  [globalZone alloc: arraysize];
    graph      = [Histogram create: globalZone];
    wintitle   = "Population Structure";
    xaxislabel = "";
    yaxislabel = "Percent";
    return self;
}

+create
{
    id result;
    result = [FuzzyGraph createBegin: globalZone];
    [result init];
    return result;					
}

-track: (unsigned*) datavar Scale: (unsigned*) onehunvar 
   Key: (char*) label Color: (char*) colorname
    // call track for each bar you want, then createEnd
{
    if (numlines == TracesPerChunk) {
        geckoError(InvalidCombination, 
            "FuzzyGraph:  max %d traces implemented.\n", TracesPerChunk);
        return self; 
    }

    data[numlines]   	 = datavar;
    onehundred[numlines] = onehunvar;
    labels[numlines] 	 = label;
    colors[numlines] 	 = colorname;
    if ((*onehundred[numlines]) == 0)
        toplot[numlines] = 0.0;
    else
        toplot[numlines] = (*data[numlines]) * 100.0 / (*onehundred[numlines]) ;
    numlines++;
    
    return self;        
}

-setWinTitle: (char*) label
    // optional
{
    wintitle   = label;
    return self;
}

-setXAxisLabel: (char*) label
    // optional
{
    xaxislabel = label;
    return self;
}

-setYAxisLabel: (char*) label
    // optional
{
    yaxislabel = label;
    return self;
}

-createEnd 
{
    if (numlines == 0) {
        geckoError(InvalidCombination,
            "FuzzyGraph not initialized properly--no lines.\n");
        return self;
    }
    
    [graph setNumPoints: numlines Labels: labels Colors: colors];
    [graph setTitle: wintitle];
    [graph setAxisLabelsX: xaxislabel Y: yaxislabel];
    [graph pack];

    return self;
}

-step 
    // update each bar
{
    int i;
    
    for (i=0; i < numlines; i++) {
        if ((*onehundred[i]) == 0)
            toplot[i] = 0.0;
        else
            toplot[i] = (*data[i]) * 100.0 / (*onehundred[i]) ;
    }
    [graph drawHistoWithDouble: toplot];
    	  
    return self;
}

*/
